package it.amattioli.dominate.hibernate.filters;

import java.io.Serializable;
import java.util.Collection;

import it.amattioli.dominate.Entity;
import it.amattioli.dominate.Repository;
import it.amattioli.dominate.RepositoryRegistry;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.criterion.Restrictions;

public class EntityFilter<I extends Serializable,T extends Entity<I>> implements ValueListAttributeFilter<T> {
	private String alias;
    private String attributeName;
    private T value;
    private Class<T> entity;

    public EntityFilter(String attributeName, Class<T> entity) {
        this.attributeName = attributeName;
        this.entity = entity;
    }
    
    public EntityFilter(String alias, String attributeName, Class<T> entity) {
        this.attributeName = attributeName;
        this.alias = alias;
        this.entity = entity;
    }

    public T get() {
        return value;
    }

    public void set(T value) {
        this.value = value;
    }
    
    public Collection<T> getValues() {
    	Repository<I, T> rep = RepositoryRegistry.instance().getRepository(entity);
    	return rep.list();
	}

    protected boolean wasSet() {
        return get() != null;
    }

    public void addCriteria(Criteria crit) {
        if (wasSet()) {
        	crit.add(Restrictions.eq(attributeName, get()));
        }
    }

    public void addHqlCondition(StringBuffer buffer) {
        if (wasSet()) {
            if (buffer.length() == 0) {
                buffer.append("where ");
            } else {
                buffer.append("and ");
            }
            if (alias != null) {
            	buffer.append(alias).append('.');
            }
            buffer.append(attributeName)
                  .append(" = :")
                  .append(attributeName.replace('.', '_'))
                  .append(" ");
        }
    }

    public void setHqlParam(Query query) {
        if (wasSet()) {
        	T param = get();
            query.setParameter(attributeName.replace('.', '_'), param);
        }
    }

}
