package it.amattioli.dominate.hibernate.filters;

import it.amattioli.dominate.Entity;
import it.amattioli.dominate.Repository;
import it.amattioli.dominate.RepositoryRegistry;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.criterion.Restrictions;

public class MultipleEntityFilter<I extends Serializable,T extends Entity<I>> implements HibernateAttributeFilter<Collection<T>> {
	private String alias;
    private Collection<T> values;
    private String attributeName;
    private Class<T> entity;
    
    public MultipleEntityFilter(String attributeName, Class<T> entity) {
        this.attributeName = attributeName;
        this.entity = entity;
    }
    
    public MultipleEntityFilter(String alias, String attributeName, Class<T> entity) {
        this.attributeName = attributeName;
        this.alias = alias;
        this.entity = entity;
    }

    public Collection<T> get() {
        return values;
    }

    public void set(Collection<T> values) {
        this.values = new ArrayList<T>(values);
    }
    
    public Collection<T> getValues() {
    	Repository<I, T> rep = RepositoryRegistry.instance().getRepository(entity);
    	return rep.list();
	}

    private boolean wasSet() {
        return get() != null && !get().isEmpty();
    }

    public void addCriteria(Criteria crit) {
        if (wasSet()) {
            crit.add(Restrictions.in(attributeName, get()));
        }
    }

    public void addHqlCondition(StringBuffer buffer) {
        if (wasSet()) {
            if (buffer.length() == 0) {
                buffer.append("where ");
            } else {
                buffer.append("and ");
            }
            if (alias != null) {
            	buffer.append(alias).append('.');
            }
            buffer.append(attributeName)
                  .append(" in ( ");
            for (int i = 0; i < get().size(); i++) {
                buffer.append(":").append(attributeName.replace('.', '_')).append(i);
                if (i != get().size() - 1) {
                    buffer.append(" , ");
                }
            }
            buffer.append(" )");
        }
    }

    public void setHqlParam(Query query) {
        if (wasSet()) {
            int i = 0;
            for (T value : get()) {
                query.setParameter(attributeName.replace('.', '_') + (i++), value);
            }
        }
    }

}
