package it.amattioli.dominate.hibernate.types;

import it.amattioli.dominate.Entity;
import it.amattioli.dominate.RepositoryRegistry;
import it.amattioli.dominate.lazy.LazyEntity;

import java.io.Serializable; 
import java.sql.PreparedStatement; 
import java.sql.ResultSet; 
import java.sql.SQLException; 
import java.sql.Types; 
import java.util.Properties;

import org.hibernate.HibernateException; 
import org.hibernate.usertype.ParameterizedType;
import org.hibernate.usertype.UserType; 
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ExternalEntityUserType<I extends Serializable, T extends Entity<I>> implements UserType, ParameterizedType {
	public static final String ENTITY_CLASS_PARAM = "entityClass";
	public static final String ID_TYPE_PARAM = "idType";
	public static final String LAZY_PARAM = "lazy";
	private static final Logger logger = LoggerFactory.getLogger(ExternalEntityUserType.class);
	
	 
    private Class<T> clazz = null;
    private int sqlType = Types.VARCHAR;
    private boolean lazy = true;
    
    public ExternalEntityUserType() {
    	
    }
    
    protected ExternalEntityUserType(Class<T> c, int sqlType) { 
        this.clazz = c; 
        this.sqlType = sqlType;
    } 
     
    public int[] sqlTypes() { 
        return new int[] { sqlType }; 
    } 
 
    public Class<T> returnedClass() { 
        return clazz; 
    } 
 
    public Object nullSafeGet(ResultSet resultSet, String[] names, Object owner) throws HibernateException, SQLException { 
        I id = (I)resultSet.getString(names[0]); 
        T result = null; 
        if (!resultSet.wasNull()) {
        	if (lazy) {
        		result = LazyEntity.newInstance(clazz, id);
        	} else {
        		result = RepositoryRegistry.instance().getRepository(clazz).get(id);
        	}
        } 
        return result; 
    } 
 
    public void nullSafeSet(PreparedStatement preparedStatement, Object value, int index) throws HibernateException, SQLException { 
        if (null == value) { 
        	logger.debug("binding null to parameter "+index);
            preparedStatement.setNull(index, sqlTypes()[0]); 
        } else {
        	I toBeSet = ((T)value).getId();
        	logger.debug("binding " + toBeSet + " to parameter " + index);
        	preparedStatement.setObject(index, toBeSet, sqlTypes()[0]);
        } 
    } 
 
    public Object deepCopy(Object value) throws HibernateException{ 
        return value; 
    } 
 
    public boolean isMutable() { 
        return false; 
    } 
 
    public Object assemble(Serializable cached, Object owner) throws HibernateException {  
         return cached;
    } 

    public Serializable disassemble(Object value) throws HibernateException { 
        return (Serializable)value; 
    } 
 
    public Object replace(Object original, Object target, Object owner) throws HibernateException { 
        return original; 
    }
    
    public int hashCode(Object x) throws HibernateException { 
        return x.hashCode(); 
    }
    
    public boolean equals(Object x, Object y) throws HibernateException { 
    	if (x == y) {
    		return true;
    	}
    	if (null == x || null == y) {
    		return false;
    	}
    	return x.equals(y);
    }

	@Override
	public void setParameterValues(Properties props) {
		if (props != null) {
			String s =(String)props.get(ENTITY_CLASS_PARAM);
			try {
				clazz = (Class<T>)Class.forName(s);
			} catch (ClassNotFoundException e) {
				throw new IllegalArgumentException("entityClass "+s+" not found");
			}
			String t = (String)props.get(ID_TYPE_PARAM);
			try {
				sqlType = Types.class.getField(t).getInt(null);
			} catch (SecurityException e) {
				throw new IllegalArgumentException("sqlType "+t+" not found");
			} catch (IllegalAccessException e) {
				throw new IllegalArgumentException("sqlType "+t+" not found");
			} catch (NoSuchFieldException e) {
				throw new IllegalArgumentException("sqlType "+t+" not found");
			}
			String l = (String)props.get(LAZY_PARAM);
			lazy = Boolean.parseBoolean(l);
		}
	}

}
