package it.amattioli.dominate.specifications;

import it.amattioli.dominate.Entity;
import it.amattioli.dominate.Specification;

public abstract class ChainedSpecification<T extends Entity<?>> extends AbstractSpecification<T> {
	private Specification<T> nextInChain;
	
	public ChainedSpecification() {
		this.nextInChain = null;
	}
	
	public ChainedSpecification(Specification<T> nextInChain) {
		this.nextInChain = nextInChain;
	}
	
	protected Specification<T> getNextInChain() {
		return nextInChain;
	}
	
	public boolean supportsAssembler(Assembler assembler) {
		return itselfSupportsAssembler(assembler) 
		    || (nextInChain != null && nextInChain.supportsAssembler(assembler));
	}
	
	public abstract boolean itselfSupportsAssembler(Assembler assembler);

	public void assembleQuery(Assembler assembler) {
		if (!wasSet() && !isSatisfiedIfNotSet()) {
			assembler.noResults();
		} else {
			if (itselfSupportsAssembler(assembler)) {
				itselfAssembleQuery(assembler);
			} else {
				chainAssembler(assembler);
			}
		}
	}
	
	public abstract void itselfAssembleQuery(Assembler assembler);
	
	protected void chainAssembler(Assembler assembler) {
		if (nextInChain != null) {
			nextInChain.assembleQuery(assembler);
		}
	}

}
