package it.amattioli.dominate.specifications;

import java.io.Serializable;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Collection;

import org.apache.commons.beanutils.PropertyUtils;

import it.amattioli.dominate.Entity;
import it.amattioli.dominate.RepositoryRegistry;

public abstract class EntitySpecification<T extends Entity<?>, I extends Serializable, U extends Entity<I>> extends ChainedSpecification<T> {
	private String propertyName;
	private U value;
	private Class<U> entityClass;
	
	public static <T extends Entity<?>, I extends Serializable, U extends Entity<I>> EntitySpecification<T, I, U> newInstance(String propertyName, Class<U> entityClass) {
		EntitySpecification<T, I, U> result = null;
		for (Class<? extends EntitySpecification> curr: SpecificationsConfig.getSpecificationClasses(EntitySpecification.class)) {
			Constructor<? extends EntitySpecification> c;
			try {
				c = curr.getConstructor(String.class, Class.class, EntitySpecification.class);
			} catch (SecurityException e) {
				throw new RuntimeException(e);
			} catch (NoSuchMethodException e) {
				throw new RuntimeException(e);
			}
			try {
				result = (EntitySpecification<T, I, U>)c.newInstance(propertyName, entityClass, result);
			} catch (IllegalArgumentException e) {
				throw new RuntimeException(e);
			} catch (InstantiationException e) {
				throw new RuntimeException(e);
			} catch (IllegalAccessException e) {
				throw new RuntimeException(e);
			} catch (InvocationTargetException e) {
				throw new RuntimeException(e);
			}
		}
		return result;
	}
	
	public EntitySpecification(String propertyName, Class<U> entityClass) {
		this.propertyName = propertyName;
		this.entityClass = entityClass;
	}
	
	public EntitySpecification(String propertyName, Class<U> entityClass, EntitySpecification<T, I, U> chained) {
		super(chained);
		this.propertyName = propertyName;
		this.entityClass = entityClass;
	}

	protected String getPropertyName() {
		return propertyName;
	}
	
	public void setValue(U value) {
		this.value = value;
		if (getNextInChain() != null) {
			((EntitySpecification<T, I, U>)getNextInChain()).setValue(value);
		}
	}

	public U getValue() {
		return value;
	}
	
	public void setId(I id) {
		setValue(RepositoryRegistry.instance().getRepository(entityClass).get(id));
	}
	
	public Collection<U> getAdmittedValues() {
    	return RepositoryRegistry.instance().getRepository(entityClass).list();
	}
	
	@Override
	public boolean isSatisfiedBy(T entity) {
		if (!wasSet()) {
			return isSatisfiedIfNotSet();
		}
		try {
			U propertyValue = (U)PropertyUtils.getProperty(entity, getPropertyName());
			return getValue().equals(propertyValue);
		} catch (IllegalAccessException e) {
			throw new RuntimeException();
		} catch (InvocationTargetException e) {
			throw new RuntimeException();
		} catch (NoSuchMethodException e) {
			throw new RuntimeException();
		}
	}

	public boolean wasSet() {
		return getValue() != null;
	}
}
