package it.amattioli.dominate.specifications;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;
import java.util.Collection;

import org.apache.commons.beanutils.PropertyUtils;

import it.amattioli.dominate.Entity;

public abstract class EnumSpecification<T extends Entity<?>, U extends Enum<U>> extends ChainedSpecification<T> {
	private String propertyName;
	private U value;
	private Class<U> enumClass;
	
	public static <T extends Entity<?>, U extends Enum<U>> EnumSpecification<T, U> newInstance(String propertyName, Class<U> enumClass) {
		EnumSpecification<T, U> result = null;
		for (Class<? extends EnumSpecification> curr: SpecificationsConfig.getSpecificationClasses(EnumSpecification.class)) {
			Constructor<? extends EnumSpecification> c;
			try {
				c = curr.getConstructor(String.class, Class.class, EnumSpecification.class);
			} catch (SecurityException e) {
				throw new RuntimeException(e);
			} catch (NoSuchMethodException e) {
				throw new RuntimeException(e);
			}
			try {
				result = (EnumSpecification<T, U>)c.newInstance(propertyName, enumClass, result);
			} catch (IllegalArgumentException e) {
				throw new RuntimeException(e);
			} catch (InstantiationException e) {
				throw new RuntimeException(e);
			} catch (IllegalAccessException e) {
				throw new RuntimeException(e);
			} catch (InvocationTargetException e) {
				throw new RuntimeException(e);
			}
		}
		return result;
	}
	
	public EnumSpecification(String propertyName, Class<U> enumClass) {
		this.propertyName = propertyName;
		this.enumClass = enumClass;
	}
	
	public EnumSpecification(String propertyName, Class<U> enumClass, EnumSpecification<T, U> chained) {
		super(chained);
		this.propertyName = propertyName;
		this.enumClass = enumClass;
	}

	protected String getPropertyName() {
		return propertyName;
	}

	public void setValue(U value) {
		this.value = value;
		if (getNextInChain() != null) {
			((EnumSpecification<T,U>)getNextInChain()).setValue(value);
		}
	}

	public U getValue() {
		return value;
	}

	@SuppressWarnings("unchecked")
    public Collection<T> getAdmittedValues() {
        try {
            return Arrays.asList((T[])enumClass.getMethod("values").invoke(null));
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new RuntimeException(e);
        } catch (NoSuchMethodException e) {
            throw new RuntimeException(e);
        }
    }
	
	@Override
	public boolean isSatisfiedBy(T entity) {
		if (!wasSet()) {
			return isSatisfiedIfNotSet();
		}
		try {
			U propertyValue = (U)PropertyUtils.getProperty(entity, getPropertyName());
			return getValue().equals(propertyValue);
		} catch (IllegalAccessException e) {
			throw new RuntimeException();
		} catch (InvocationTargetException e) {
			throw new RuntimeException();
		} catch (NoSuchMethodException e) {
			throw new RuntimeException();
		}
	}

	public boolean wasSet() {
		return getValue() != null;
	}

}