package it.amattioli.dominate.specifications;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;

import it.amattioli.dominate.Entity;

public abstract class ExclusionListSpecification<T extends Entity<?>> extends ChainedSpecification<T> {
	private Collection<T> exclusionList = new HashSet<T>();
	
	public static <T extends Entity<?>> ExclusionListSpecification<T> newInstance() {
		ExclusionListSpecification<T> result = null;
		for (Class<? extends ExclusionListSpecification> curr: SpecificationsConfig.getSpecificationClasses(ExclusionListSpecification.class)) {
			Constructor<? extends ExclusionListSpecification> c;
			try {
				c = curr.getConstructor(ExclusionListSpecification.class);
			} catch (SecurityException e) {
				throw new RuntimeException(e);
			} catch (NoSuchMethodException e) {
				throw new RuntimeException(e);
			}
			try {
				result = (ExclusionListSpecification<T>)c.newInstance(result);
			} catch (IllegalArgumentException e) {
				throw new RuntimeException(e);
			} catch (InstantiationException e) {
				throw new RuntimeException(e);
			} catch (IllegalAccessException e) {
				throw new RuntimeException(e);
			} catch (InvocationTargetException e) {
				throw new RuntimeException(e);
			}
		}
		return result;
	}
	
	public ExclusionListSpecification() {
		
	}
	
	public ExclusionListSpecification(ExclusionListSpecification<T> chained) {
		super(chained);
	}
	
	public Collection<T> getExclusionList() {
		return Collections.unmodifiableCollection(exclusionList);
	}
	
	public void addToExclusionList(T entity) {
		exclusionList.add(entity);
		if (getNextInChain() != null) {
			((ExclusionListSpecification<T>)getNextInChain()).addToExclusionList(entity);
		}
	}
	
	public void removeFromExclusionList(T entity) {
		exclusionList.remove(entity);
		if (getNextInChain() != null) {
			((ExclusionListSpecification<T>)getNextInChain()).removeFromExclusionList(entity);
		}
	}
	
	@Override
	public boolean isSatisfiedBy(T entity) {
		if (!wasSet()) {
			return isSatisfiedIfNotSet();
		}
		return !exclusionList.contains(entity);
	}
	
	public boolean wasSet() {
		return !getExclusionList().isEmpty();
	}
}
