package it.amattioli.dominate.specifications;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import org.apache.commons.beanutils.PropertyUtils;

import it.amattioli.dominate.Entity;

public abstract class LongSpecification<T extends Entity<?>> extends ChainedSpecification<T> {
	private TotalOrderComparisonType comparisonType = TotalOrderComparisonType.EQUAL;
	private String propertyName;
	private Long value;
	
	public static <T extends Entity<?>> LongSpecification<T> newInstance(String propertyName) {
		LongSpecification<T> result = null;
		for (Class<? extends LongSpecification> curr: SpecificationsConfig.getSpecificationClasses(LongSpecification.class)) {
			Constructor<? extends LongSpecification> c;
			try {
				c = curr.getConstructor(String.class, LongSpecification.class);
			} catch (SecurityException e) {
				throw new RuntimeException(e);
			} catch (NoSuchMethodException e) {
				throw new RuntimeException(e);
			}
			try {
				result = (LongSpecification<T>)c.newInstance(propertyName, result);
			} catch (IllegalArgumentException e) {
				throw new RuntimeException(e);
			} catch (InstantiationException e) {
				throw new RuntimeException(e);
			} catch (IllegalAccessException e) {
				throw new RuntimeException(e);
			} catch (InvocationTargetException e) {
				throw new RuntimeException(e);
			}
		}
		return result;
	}
	
	public LongSpecification(String propertyName) {
		this.propertyName = propertyName;
	}
	
	public LongSpecification(String propertyName, LongSpecification<T> chained) {
		super(chained);
		this.propertyName = propertyName;
	}

	protected String getPropertyName() {
		return propertyName;
	}

	public void setValue(Long value) {
		this.value = value;
		if (getNextInChain() != null) {
			((LongSpecification<T>)getNextInChain()).setValue(value);
		}
	}

	public Long getValue() {
		return value;
	}

	public TotalOrderComparisonType getComparisonType() {
		return comparisonType;
	}

	public void setComparisonType(TotalOrderComparisonType comparisonType) {
		this.comparisonType = comparisonType;
		if (getNextInChain() != null) {
			((LongSpecification<T>)getNextInChain()).setComparisonType(comparisonType);
		}
	}

	@Override
	public boolean isSatisfiedBy(T entity) {
		if (!wasSet()) {
			return isSatisfiedIfNotSet();
		}
		try {
			Long propertyValue = (Long)PropertyUtils.getProperty(entity, getPropertyName());
			return comparisonType.isSatisfiedBy(propertyValue, getValue());
		} catch (IllegalAccessException e) {
			throw new RuntimeException();
		} catch (InvocationTargetException e) {
			throw new RuntimeException();
		} catch (NoSuchMethodException e) {
			throw new RuntimeException();
		}
	}

	public boolean wasSet() {
		return getValue() != null;
	}

}