package it.amattioli.dominate.specifications;

import java.io.Serializable;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Collection;

import org.apache.commons.beanutils.PropertyUtils;

import it.amattioli.dominate.Entity;
import it.amattioli.dominate.RepositoryRegistry;

public abstract class MultipleEntitySpecification<T extends Entity<?>, I extends Serializable, U extends Entity<I>> extends ChainedSpecification<T> {
	private String propertyName;
	private Collection<U> value;
	private Class<U> entityClass;
	
	public static <T extends Entity<?>, I extends Serializable, U extends Entity<I>> MultipleEntitySpecification<T, I, U> newInstance(String propertyName, Class<U> entityClass) {
		MultipleEntitySpecification<T, I, U> result = null;
		for (Class<? extends MultipleEntitySpecification> curr: SpecificationsConfig.getSpecificationClasses(MultipleEntitySpecification.class)) {
			Constructor<? extends MultipleEntitySpecification> c;
			try {
				c = curr.getConstructor(String.class, Class.class, MultipleEntitySpecification.class);
			} catch (SecurityException e) {
				throw new RuntimeException(e);
			} catch (NoSuchMethodException e) {
				throw new RuntimeException(e);
			}
			try {
				result = (MultipleEntitySpecification<T, I, U>)c.newInstance(propertyName, entityClass, result);
			} catch (IllegalArgumentException e) {
				throw new RuntimeException(e);
			} catch (InstantiationException e) {
				throw new RuntimeException(e);
			} catch (IllegalAccessException e) {
				throw new RuntimeException(e);
			} catch (InvocationTargetException e) {
				throw new RuntimeException(e);
			}
		}
		return result;
	}
	
	public MultipleEntitySpecification(String propertyName, Class<U> entityClass) {
		this.propertyName = propertyName;
		this.entityClass = entityClass;
	}
	
	public MultipleEntitySpecification(String propertyName, Class<U> entityClass, MultipleEntitySpecification<T, I, U> chained) {
		super(chained);
		this.propertyName = propertyName;
		this.entityClass = entityClass;
	}

	protected String getPropertyName() {
		return propertyName;
	}
	
	public void setValue(Collection<U> value) {
		this.value = value;
		if (getNextInChain() != null) {
			((MultipleEntitySpecification<T, I, U>)getNextInChain()).setValue(value);
		}
	}

	public Collection<U> getValue() {
		return value;
	}
	
	public Collection<U> getAdmittedValues() {
    	return RepositoryRegistry.instance().getRepository(entityClass).list();
	}
	
	@Override
	public boolean isSatisfiedBy(T entity) {
		if (!wasSet()) {
			return isSatisfiedIfNotSet();
		}
		try {
			String propertyValue = (String)PropertyUtils.getProperty(entity, getPropertyName());
			return getValue().contains(propertyValue);
		} catch (IllegalAccessException e) {
			throw new RuntimeException();
		} catch (InvocationTargetException e) {
			throw new RuntimeException();
		} catch (NoSuchMethodException e) {
			throw new RuntimeException();
		}
	}

	public boolean wasSet() {
		return getValue() != null && !getValue().isEmpty();
	}
}
