package it.amattioli.dominate.specifications;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;
import java.util.Collection;

import org.apache.commons.beanutils.PropertyUtils;
import org.hibernate.usertype.UserType;

import it.amattioli.dominate.Entity;
import it.amattioli.dominate.specifications.dflt.DefaultMultipleEnumSpecification;
import it.amattioli.dominate.specifications.hibernate.CriteriaMultipleEnumSpecification;
import it.amattioli.dominate.specifications.hibernate.HqlMultipleEnumSpecification;

public abstract class MultipleEnumSpecification<T extends Entity<?>, U extends Enum<U>> extends ChainedSpecification<T> {
	private String propertyName;
	private Collection<U> value;
	private Class<U> enumClass;
	
	public static <T extends Entity<?>, U extends Enum<U>> MultipleEnumSpecification<T, U> newInstance(String propertyName, Class<U> enumClass) {
		MultipleEnumSpecification<T, U> result = null;
		for (Class<? extends MultipleEnumSpecification> curr: SpecificationsConfig.getSpecificationClasses(MultipleEnumSpecification.class)) {
			Constructor<? extends MultipleEnumSpecification> c;
			try {
				c = curr.getConstructor(String.class, Class.class, MultipleEnumSpecification.class);
			} catch (SecurityException e) {
				throw new RuntimeException(e);
			} catch (NoSuchMethodException e) {
				throw new RuntimeException(e);
			}
			try {
				result = (MultipleEnumSpecification<T, U>)c.newInstance(propertyName, enumClass, result);
			} catch (IllegalArgumentException e) {
				throw new RuntimeException(e);
			} catch (InstantiationException e) {
				throw new RuntimeException(e);
			} catch (IllegalAccessException e) {
				throw new RuntimeException(e);
			} catch (InvocationTargetException e) {
				throw new RuntimeException(e);
			}
		}
		return result;
	}
	
	public static <T extends Entity<?>, U extends Enum<U>> MultipleEnumSpecification<T, U> newInstance(String propertyName, Class<U> enumClass, Class<? extends UserType> hibernateType) {
		MultipleEnumSpecification<T, U> h = new HqlMultipleEnumSpecification<T, U>(propertyName, enumClass, hibernateType);
		MultipleEnumSpecification<T, U> c = new CriteriaMultipleEnumSpecification<T, U>(propertyName, enumClass, h);
		MultipleEnumSpecification<T, U> result = new DefaultMultipleEnumSpecification<T, U>(propertyName, enumClass, c);
		return result;
	}
	
	public MultipleEnumSpecification(String propertyName, Class<U> enumClass) {
		this.propertyName = propertyName;
		this.enumClass = enumClass;
	}
	
	public MultipleEnumSpecification(String propertyName, Class<U> enumClass, MultipleEnumSpecification<T, U> chained) {
		super(chained);
		this.propertyName = propertyName;
		this.enumClass = enumClass;
	}

	protected String getPropertyName() {
		return propertyName;
	}

	public void setValue(Collection<U> value) {
		this.value = value;
		if (getNextInChain() != null) {
			((MultipleEnumSpecification<T, U>)getNextInChain()).setValue(value);
		}
	}

	public Collection<U> getValue() {
		return value;
	}

	@SuppressWarnings("unchecked")
    public Collection<T> getAdmittedValues() {
        try {
            return Arrays.asList((T[])enumClass.getMethod("values").invoke(null));
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new RuntimeException(e);
        } catch (NoSuchMethodException e) {
            throw new RuntimeException(e);
        }
    }
	
	@Override
	public boolean isSatisfiedBy(T entity) {
		if (!wasSet()) {
			return isSatisfiedIfNotSet();
		}
		try {
			U propertyValue = (U)PropertyUtils.getProperty(entity, getPropertyName());
			return getValue().contains(propertyValue);
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e);
		} catch (InvocationTargetException e) {
			throw new RuntimeException(e);
		} catch (NoSuchMethodException e) {
			throw new RuntimeException(e);
		}
	}

	public boolean wasSet() {
		return getValue() != null && !getValue().isEmpty();
	}

}