package it.amattioli.dominate.specifications;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import org.apache.commons.beanutils.PropertyUtils;

import it.amattioli.dominate.Entity;

public abstract class ObjectSpecification<T extends Entity<?>> extends ChainedSpecification<T> {
	private String propertyName;
	private Object value;

	public static <T extends Entity<?>> ObjectSpecification<T> newInstance(String propertyName) {
		ObjectSpecification<T> result = null;
		for (Class<? extends ObjectSpecification> curr: SpecificationsConfig.getSpecificationClasses(ObjectSpecification.class)) {
			Constructor<? extends ObjectSpecification> c;
			try {
				c = curr.getConstructor(String.class, ObjectSpecification.class);
			} catch (SecurityException e) {
				throw new RuntimeException(e);
			} catch (NoSuchMethodException e) {
				throw new RuntimeException(e);
			}
			try {
				result = (ObjectSpecification<T>)c.newInstance(propertyName, result);
			} catch (IllegalArgumentException e) {
				throw new RuntimeException(e);
			} catch (InstantiationException e) {
				throw new RuntimeException(e);
			} catch (IllegalAccessException e) {
				throw new RuntimeException(e);
			} catch (InvocationTargetException e) {
				throw new RuntimeException(e);
			}
		}
		return result;
	}

	public ObjectSpecification(String propertyName) {
		this.propertyName = propertyName;
	}
	
	public ObjectSpecification(String propertyName, ObjectSpecification<T> chained) {
		super(chained);
		this.propertyName = propertyName;
	}

	protected String getPropertyName() {
		return propertyName;
	}
	
	public void setValue(Object value) {
		this.value = value;
		if (getNextInChain() != null) {
			((ObjectSpecification<T>)getNextInChain()).setValue(value);
		}
	}

	public Object getValue() {
		return value;
	}
	
	@Override
	public boolean isSatisfiedBy(T entity) {
		if (!wasSet()) {
			return isSatisfiedIfNotSet();
		}
		try {
			Object propertyValue = PropertyUtils.getProperty(entity, getPropertyName());
			return getValue().equals(propertyValue);
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e);
		} catch (InvocationTargetException e) {
			throw new RuntimeException(e);
		} catch (NoSuchMethodException e) {
			throw new RuntimeException(e);
		}
	}

	public boolean wasSet() {
		return getValue() != null;
	}
}
