package it.amattioli.dominate.specifications;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Properties;

import it.amattioli.dominate.Specification;

public class SpecificationsConfig {
	private static final String SPECIFICATIONS_PROPERTIES = "specifications.properties";
	private static final String DEFAULT_SPECIFICATIONS_PROPERTIES = "/it/amattioli/dominate/specifications/" + SPECIFICATIONS_PROPERTIES;
	private static final String MERGING_SPECIFICATIONS_PROPERTIES = "mergeSpecifications.properties";
	private static final Collection<String> pluggedSpecifications = new ArrayList<String>();
	private static Properties configurationProperties;

	private SpecificationsConfig() {
		
	}

	public static void addSpecificationConfig(String configFileName) {
		configurationProperties = null;
		pluggedSpecifications.add(configFileName);
	}
	
	public static <T extends Specification<?>> Collection<Class<? extends T>> getSpecificationClasses(Class<T> baseClass) {
		String prefix = baseClass.getName();
		Collection<Class<? extends T>> result = new ArrayList<Class<? extends T>>();
		Properties p = getConfigurationProperties();
		for (String curr: p.stringPropertyNames()) {
			if (curr.startsWith(prefix)) {
				try {
					result.add((Class<? extends T>)Class.forName(p.getProperty(curr)));
				} catch (ClassNotFoundException e) {
					throw new RuntimeException(e);
				}
			}
		}
		return result;
	}

	private static Properties getConfigurationProperties() {
		if (configurationProperties == null) {
			configurationProperties = new Properties();
			InputStream resourceStream = SpecificationsConfig.class.getResourceAsStream(SPECIFICATIONS_PROPERTIES);
			if (resourceStream == null) {
				resourceStream = SpecificationsConfig.class.getResourceAsStream(DEFAULT_SPECIFICATIONS_PROPERTIES);
			}
			loadProperties(resourceStream);
			InputStream mergingStream = SpecificationsConfig.class.getResourceAsStream(MERGING_SPECIFICATIONS_PROPERTIES);
			if (mergingStream != null) {
				loadProperties(mergingStream);
			}
			for (String curr: pluggedSpecifications) {
				InputStream pluggedStream = SpecificationsConfig.class.getResourceAsStream(curr);
				if (pluggedStream != null) {
					loadProperties(pluggedStream);
				}
			}
		}
		return configurationProperties;
	}

	private static void loadProperties(InputStream resourceStream) {
		try {
			configurationProperties.load(resourceStream);
		} catch (IOException e) {
			throw new RuntimeException(e);
		} finally {
			try {
				if (resourceStream != null) {
					resourceStream.close();
				}
			} catch (IOException e) {
				throw new RuntimeException(e);
			}
		}
	}
	
	public static void reset() {
		configurationProperties = null;
		pluggedSpecifications.clear();
	}
}
