package it.amattioli.dominate.specifications;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;
import java.util.Collection;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang.StringUtils;

import it.amattioli.dominate.Entity;
import it.amattioli.dominate.hibernate.filters.ComparisonType;
import static it.amattioli.dominate.hibernate.filters.ComparisonType.*;

public abstract class StringSpecification<T extends Entity<?>> extends ChainedSpecification<T> {
	private String propertyName;
	private String value;
	private ComparisonType comparisonType = ComparisonType.STARTS;
	
	public static <T extends Entity<?>> StringSpecification<T> newInstance(String propertyName) {
		StringSpecification<T> result = null;
		for (Class<? extends StringSpecification> curr: SpecificationsConfig.getSpecificationClasses(StringSpecification.class)) {
			Constructor<? extends StringSpecification> c;
			try {
				c = curr.getConstructor(String.class, StringSpecification.class);
			} catch (SecurityException e) {
				throw new RuntimeException(e);
			} catch (NoSuchMethodException e) {
				throw new RuntimeException(e);
			}
			try {
				result = (StringSpecification<T>)c.newInstance(propertyName, result);
			} catch (IllegalArgumentException e) {
				throw new RuntimeException(e);
			} catch (InstantiationException e) {
				throw new RuntimeException(e);
			} catch (IllegalAccessException e) {
				throw new RuntimeException(e);
			} catch (InvocationTargetException e) {
				throw new RuntimeException(e);
			}
		}
		return result;
	}
	
	public StringSpecification(String propertyName) {
		this.propertyName = propertyName;
	}
	
	public StringSpecification(String propertyName, StringSpecification<T> chained) {
		super(chained);
		this.propertyName = propertyName;
	}

	protected String getPropertyName() {
		return propertyName;
	}

	public void setValue(String value) {
		this.value = value;
		if (getNextInChain() != null) {
			((StringSpecification<T>)getNextInChain()).setValue(value);
		}
	}

	public String getValue() {
		return value;
	}
	
	public ComparisonType getComparisonType() {
        return comparisonType;
    }

    public void setComparisonType(ComparisonType filterType) {
        this.comparisonType = filterType;
        if (getNextInChain() != null) {
        	((StringSpecification<T>)getNextInChain()).setComparisonType(filterType);
        }
    }

    public void setComparisonType(String filterType) {
        setComparisonType(ComparisonType.valueOf(filterType));
    }

    public Collection<ComparisonType> getComparisonTypeValues() {
        return Arrays.asList(ComparisonType.values());
    }

	@Override
	public boolean isSatisfiedBy(T entity) {
		if (!wasSet()) {
			return isSatisfiedIfNotSet();
		}
		try {
			String propertyValue = (String)PropertyUtils.getProperty(entity, getPropertyName());
			return getComparisonType().equals(EXACT) && StringUtils.equals(getValue(), propertyValue)
			    || getComparisonType().equals(STARTS) && StringUtils.startsWith(propertyValue, getValue())
		        || getComparisonType().equals(CONTAINS) && StringUtils.contains(propertyValue, getValue());
		} catch (IllegalAccessException e) {
			throw new RuntimeException();
		} catch (InvocationTargetException e) {
			throw new RuntimeException();
		} catch (NoSuchMethodException e) {
			throw new RuntimeException();
		}
	}

	public boolean wasSet() {
		return getValue() != null && !getValue().equals("");
	}

}